local config = require("gemini.config")
local M = {}

-- Store chat state
local state = {
    bufnr = nil,
    winnr = nil,
    current_context = nil,
    conversation_history = {}
}

local function setup_chat_highlighting(bufnr)
    vim.api.nvim_buf_set_option(bufnr, 'syntax', '')
    pcall(vim.treesitter.start, bufnr, "markdown")

    vim.cmd([[
        highlight default GeminiUser guifg=#EBCB8B gui=bold
        highlight default GeminiSeparator guifg=#616E88 gui=bold
        syntax match GeminiUser /^User:.*$/
        syntax match GeminiSeparator /^━━━━━━━━━━━━━━━━━━━━━━━━━━$/
    ]])
end

local function setup_buffer_options(bufnr)
    -- Set strict buffer options
    local options = {
        buftype = 'nofile',
        filetype = 'markdown',
        buflisted = false,
        swapfile = false,
        bufhidden = 'wipe',
        modifiable = false,
        readonly = true,
        undolevels = -1,
        list = false,
        number = false,
        relativenumber = false,
        modified = false,
        foldmethod = 'manual',
        foldlevel = 99,
        textwidth = 0,
        wrapmargin = 0,
        expandtab = true,
        autoindent = false,
        smartindent = false,
    }

    for option, value in pairs(options) do
        vim.api.nvim_buf_set_option(bufnr, option, value)
    end

    -- Clear buffer name to prevent file operations
    vim.api.nvim_buf_set_name(bufnr, '')
end

local function setup_buffer_autocmds(bufnr)
    local augroup = vim.api.nvim_create_augroup('GeminiChatBuffer', { clear = true })
    
    -- Block ALL buffer events that could load files
    local blocked_events = {
        'BufReadCmd', 'FileReadCmd', 'BufWriteCmd', 'FileWriteCmd',
        'BufReadPre', 'BufReadPost', 'BufWrite', 'BufWritePre', 'BufWritePost',
        'FileReadPre', 'FileReadPost', 'FileWritePre', 'FileWritePost',
        'BufAdd', 'BufCreate', 'BufDelete', 'BufWipeout',
        'BufNew', 'BufNewFile', 'BufRead', 'BufEnter', 'BufLeave',
        'FileType', 'BufFilePre', 'BufFilePost',
        'FileChangedShell', 'FileChangedShellPost',
        'FileChangedRO', 'FileAppendCmd', 'FileAppendPre', 'FileAppendPost',
        'FilterReadPre', 'FilterReadPost', 'FocusGained', 'FocusLost'
    }
    
    for _, event in ipairs(blocked_events) do
        vim.api.nvim_create_autocmd(event, {
            group = augroup,
            buffer = bufnr,
            callback = function()
                vim.notify('File operations are not allowed in the chat window', vim.log.levels.WARN)
                return true  -- Prevent the default handler
            end
        })
    end

    -- Override buffer behavior
    vim.api.nvim_create_autocmd('BufEnter', {
        group = augroup,
        buffer = bufnr,
        callback = function()
            -- Force buffer settings on every enter
            vim.opt_local.modifiable = false
            vim.opt_local.readonly = true
            vim.opt_local.swapfile = false
            vim.opt_local.buftype = 'nofile'
            vim.opt_local.buflisted = false
            
            -- Disable file loading capabilities
            vim.bo[bufnr].buftype = 'nofile'
            vim.bo[bufnr].filetype = 'markdown'
            
            -- Clear file name to prevent file operations
            vim.api.nvim_buf_set_name(bufnr, '')
            
            -- Return true to prevent default BufEnter handling
            return true
        end
    })

    -- Prevent any attempt to change buffer settings
    vim.api.nvim_create_autocmd('OptionSet', {
        group = augroup,
        buffer = bufnr,
        callback = function()
            vim.schedule(function()
                vim.opt_local.modifiable = false
                vim.opt_local.readonly = true
                vim.opt_local.buftype = 'nofile'
            end)
            return true
        end
    })
end

local function setup_buffer_keymaps(bufnr)
    local opts = { buffer = bufnr, nowait = true, silent = true }
    
    -- Block ALL leader key combinations - this is the key fix!
    vim.keymap.set({'n', 'v'}, '<leader>', function()
        vim.notify('Leader key commands are disabled in chat window', vim.log.levels.WARN)
        return '<Esc>'
    end, opts)
    
    -- Disable command mode
    vim.keymap.set('n', ':', function()
        vim.notify('Commands are disabled in chat window', vim.log.levels.WARN)
        return '<Esc>'
    end, opts)

    -- Only allow specific keymaps for chat functionality
    local mappings = config.options.mappings
    vim.keymap.set('n', mappings.close, function()
        vim.api.nvim_win_close(state.winnr, true)
        state.winnr = nil
    end, opts)

    vim.keymap.set('n', mappings.return_focus, function()
        vim.cmd('wincmd p')
    end, opts)

    vim.keymap.set('n', mappings.new_query, function()
        require("gemini").prompt_query(state.current_context)
    end, opts)

    -- Allow basic navigation
    local allowed_keys = {'j', 'k', '<C-d>', '<C-u>', '<C-f>', '<C-b>', 'G', 'gg'}
    for _, key in ipairs(allowed_keys) do
        vim.keymap.set('n', key, key, opts)
    end
end

function M.create_window()
    if not state.bufnr or not vim.api.nvim_buf_is_valid(state.bufnr) then
        state.bufnr = vim.api.nvim_create_buf(false, true)
        setup_buffer_options(state.bufnr)
        setup_buffer_autocmds(state.bufnr)
        setup_buffer_keymaps(state.bufnr)
    end

    local win_opts = config.options.window
    if not state.winnr or not vim.api.nvim_win_is_valid(state.winnr) then
        state.winnr = vim.api.nvim_open_win(state.bufnr, true, {
            relative = "editor",
            width = win_opts.width(),
            height = win_opts.height(),
            row = 0,
            col = vim.o.columns - win_opts.width(),
            border = win_opts.border,
            title = win_opts.title,
            title_pos = win_opts.title_pos,
            style = "minimal"
        })

        vim.api.nvim_win_set_option(state.winnr, 'wrap', true)
        vim.api.nvim_win_set_option(state.winnr, 'linebreak', true)
        vim.api.nvim_win_set_option(state.winnr, 'breakindent', true)

        setup_chat_highlighting(state.bufnr)
    end
end

function M.update_content(content, is_new_chat, is_thinking)
    -- Temporarily disable readonly and enable modifiable
    vim.api.nvim_buf_set_option(state.bufnr, 'readonly', false)
    vim.api.nvim_buf_set_option(state.bufnr, 'modifiable', true)

    local lines = vim.split(content, "\n")

    if is_thinking then
        -- For thinking message, just replace content
        vim.api.nvim_buf_set_lines(state.bufnr, 0, -1, false, lines)
    else
        -- For actual messages
        local display_lines = {}
        local current_query_line = 1
        
        -- Add all messages from history
        for i, msg in ipairs(state.conversation_history) do
            if i > 1 then
                table.insert(display_lines, "━━━━━━━━━━━━━━━━━━━━━━━━━━")
            end
            if i == #state.conversation_history - 1 then
                current_query_line = #display_lines + 1
            end
            local prefix = msg.role == "user" and "User: " or "Assistant: "
            local first_line = true
            local msg_lines = vim.split(msg.content, "\n")
            for _, line in ipairs(msg_lines) do
                if first_line then
                    table.insert(display_lines, prefix .. line)
                    first_line = false
                else
                    table.insert(display_lines, line)
                end
            end
        end

        if #display_lines > 0 then
            vim.api.nvim_buf_set_lines(state.bufnr, 0, -1, false, display_lines)
            vim.api.nvim_win_set_cursor(state.winnr, {current_query_line, 0})
            vim.cmd('normal! zt')
        end
    end

    -- Re-enable readonly and disable modifiable
    vim.api.nvim_buf_set_option(state.bufnr, 'modifiable', false)
    vim.api.nvim_buf_set_option(state.bufnr, 'readonly', true)
    vim.cmd('wincmd p')
end

function M.set_context(context)
    state.current_context = context
end

function M.clear()
    if state.bufnr and vim.api.nvim_buf_is_valid(state.bufnr) then
        vim.api.nvim_buf_set_lines(state.bufnr, 0, -1, false, {})
    end
    state.conversation_history = {}  -- Clear conversation history
end

-- Add functions to manage conversation history
function M.add_message(role, content)
    -- Remove any existing prefix from the content
    content = content:gsub("^User: ", ""):gsub("^Assistant: ", "")
    table.insert(state.conversation_history, {role = role, content = content})
end

function M.get_conversation_history()
    return state.conversation_history
end

return M