local M = {}

-- HTTP Request class
local Request = {}
Request.__index = Request

function Request.new(url, payload)
    local self = setmetatable({}, Request)
    self.url = url
    self.payload = payload
    return self
end

function Request:execute(callback)
    local stdout = vim.loop.new_pipe()
    local stderr = vim.loop.new_pipe()
    local handle
    local output = ""
    local error_output = ""

    local function cleanup()
        if handle then
            vim.loop.close(handle)
            handle = nil
        end
        stdout:close()
        stderr:close()
    end

    -- Print the curl command for debugging
    local curl_args = {
        "-s",                     -- silent mode
        "-X", "POST",            
        "-H", "Content-Type: application/json",
        "-d", self.payload,
        self.url
    }
    
    handle = vim.loop.spawn("curl", {
        args = curl_args,
        stdio = {nil, stdout, stderr}
    }, function(code)
        vim.schedule(function()
            if code ~= 0 then
                callback(nil, "Curl failed with code " .. code .. ": " .. error_output)
                return
            end
            
            -- Debug output
            if output == "" then
                callback(nil, "Empty response received")
                return
            end

            local success, decoded = pcall(vim.json.decode, output)
            if success then
                callback(decoded)
            else
                -- Include the raw output in the error for debugging
                callback(nil, string.format("JSON decode error. Raw output: %s", output))
            end
        end)
        cleanup()
    end)

    if not handle then
        vim.schedule(function()
            callback(nil, "Failed to spawn curl process")
        end)
        cleanup()
        return
    end

    stdout:read_start(function(err, data)
        if err then
            vim.schedule(function()
                callback(nil, "Error reading stdout: " .. err)
            end)
            cleanup()
            return
        end
        if data then
            output = output .. data
        end
    end)

    stderr:read_start(function(err, data)
        if err then
            vim.schedule(function()
                callback(nil, "Error reading stderr: " .. err)
            end)
            cleanup()
            return
        end
        if data then
            error_output = error_output .. data
        end
    end)
end

M.Request = Request
return M